CREATE TABLE test (
	hidden_col     INTEGER PRIMARY KEY,
	text_col       TEXT,
	password_col   TEXT,
	checkbox_col   TEXT,
	select_col     TEXT,
	radio_col      TEXT,
	radiogroup_col TEXT,
	date_col       DATE,
	not_in_form    TEXT
);
